var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { aoSend } from '../common/ao';
import { resolveTransaction } from '../common/arweave';
import { getGQLData } from '../common/gql';
import { GATEWAYS, TAGS } from '../helpers/config';
import { globalLog } from '../helpers/utils';
import { createZone, getZone, updateZone } from './zones';
export function createProfile(args, wallet, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let profileId = null;
        const tags = [
            { name: TAGS.keys.dataProtocol, value: 'Zone' },
            { name: TAGS.keys.zoneType, value: 'User' }
        ];
        const addBootTag = (key, value) => {
            if (value) {
                tags.push({ name: `${TAGS.keys.bootloader}-${key}`, value });
            }
        };
        const addImageTag = (imageKey) => __awaiter(this, void 0, void 0, function* () {
            const key = imageKey.toLowerCase();
            if (args[key]) {
                try {
                    const resolvedImage = yield resolveTransaction(args[key]);
                    addBootTag(imageKey, resolvedImage);
                }
                catch (e) {
                    if (callback)
                        callback(`Failed to resolve ${imageKey}: ${e.message}`);
                    else
                        console.error(e);
                }
            }
        });
        addBootTag('Username', args.username);
        addBootTag('DisplayName', args.displayName);
        addBootTag('Description', args.description);
        yield Promise.all([addImageTag('Thumbnail'), addImageTag('Banner')]);
        try {
            profileId = yield createZone({ tags: tags }, wallet, callback);
            if (profileId) {
                globalLog(`Profile ID: ${profileId}`);
                const registerId = yield aoSend({
                    processId: profileId,
                    wallet: wallet,
                    action: 'Register-Whitelisted-Subscriber',
                    tags: [
                        { name: 'Topics', value: JSON.stringify(['Zone-Update']) },
                        { name: 'Subscriber-Process-Id', value: 'Wl7pTf-UEp6SIIu3S5MsTX074Sg8MhCx40NuG_YEhmk' },
                    ]
                });
                console.log(`Register ID: ${registerId}`);
            }
        }
        catch (e) {
            throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error creating profile');
        }
        return profileId;
    });
}
export function updateProfile(args, profileId, wallet, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        if (profileId) {
            let data = {
                Username: args.username,
                DisplayName: args.displayName,
                Description: args.description
            };
            if (args.thumbnail) {
                try {
                    data.Thumbnail = yield resolveTransaction(args.thumbnail);
                }
                catch (e) {
                    if (callback)
                        callback(`Failed to resolve thumbnail: ${e.message}`);
                }
            }
            if (args.banner) {
                try {
                    data.Banner = yield resolveTransaction(args.banner);
                }
                catch (e) {
                    if (callback)
                        callback(`Failed to resolve banner: ${e.message}`);
                }
            }
            try {
                return yield updateZone(data, profileId, wallet);
            }
            catch (e) {
                throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error creating profile');
            }
        }
        else {
            throw new Error('No profile provided');
        }
    });
}
export function getProfileById(profileId) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d;
        try {
            const zone = yield getZone(profileId);
            if (zone && zone.Store) {
                let profile = {
                    id: profileId,
                    walletAddress: null, // TODO: Get owner
                    username: (_a = zone.Store.Username) !== null && _a !== void 0 ? _a : 'None',
                    displayName: (_b = zone.Store.DisplayName) !== null && _b !== void 0 ? _b : 'None',
                    description: (_c = zone.Store.Description) !== null && _c !== void 0 ? _c : 'None',
                };
                if (zone.Store.Thumbnail)
                    profile.thumbnail = zone.Store.Thumbnail;
                if (zone.Store.Banner)
                    profile.banner = zone.Store.Banner;
                if (zone.Assets)
                    profile.assets = zone.Assets;
                for (const [key, value] of Object.entries(zone.Store)) {
                    if (!(key in profile)) {
                        profile[key] = value;
                    }
                }
                return profile;
            }
        }
        catch (e) {
            throw new Error((_d = e.message) !== null && _d !== void 0 ? _d : 'Error fetching profile');
        }
    });
}
export function getProfileByWalletAddress(walletAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        try {
            const gqlResponse = yield getGQLData({
                gateway: GATEWAYS.goldsky,
                tags: [
                    { name: TAGS.keys.dataProtocol, values: ['Zone'] },
                    { name: TAGS.keys.zoneType, values: ['User'] },
                ],
                owners: [walletAddress]
            });
            if (((_a = gqlResponse === null || gqlResponse === void 0 ? void 0 : gqlResponse.data) === null || _a === void 0 ? void 0 : _a.length) > 0) {
                gqlResponse.data.sort((a, b) => {
                    var _a, _b, _c, _d;
                    const timestampA = (_b = (_a = a.node.block) === null || _a === void 0 ? void 0 : _a.timestamp) !== null && _b !== void 0 ? _b : 0;
                    const timestampB = (_d = (_c = b.node.block) === null || _c === void 0 ? void 0 : _c.timestamp) !== null && _d !== void 0 ? _d : 0;
                    return timestampB - timestampA;
                });
                return yield getProfileById(gqlResponse.data[0].node.id);
            }
            return { id: null };
        }
        catch (e) {
            throw new Error((_b = e.message) !== null && _b !== void 0 ? _b : 'Error fetching profile');
        }
    });
}
//# sourceMappingURL=profiles.js.map